package cordova.plugin.chief.push;

import android.annotation.TargetApi;
import android.app.Activity;
import android.app.AppOpsManager;
import android.content.Context;
import android.content.Intent;
import android.content.pm.ApplicationInfo;
import android.net.Uri;
import android.os.Build;
import android.util.Log;

import org.apache.cordova.CordovaInterface;
import org.apache.cordova.CordovaWebView;
import org.json.JSONException;

import cn.jpush.android.api.JPushInterface;
import cordova.plugin.chief.push.pushclass.SetAliasClass;
import cordova.plugin.chief.push.pushclass.SetTagsClass;

import org.apache.cordova.CordovaPlugin;
import org.apache.cordova.CallbackContext;
import org.json.JSONArray;
import org.json.JSONObject;

import java.lang.reflect.Field;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.Arrays;
import java.util.Iterator;
import java.util.List;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

/**
 * This class echoes a string called from JavaScript.
 */
public class PushInit extends CordovaPlugin {
  private static String TAG = "PushInit";
  private final static List<String> methodList =
    Arrays.asList(
      "getUuid", "init", "setAlias", "setTags", "areNotificationEnabled", "openSetingOfNotification");
  private static Activity cordovaActivity;
  private static Context cordovaContext;

  public static PushInit instance;
  public static CordovaWebView cordovaWebView;
  //异步线程池
  private ExecutorService threadPool = Executors.newFixedThreadPool(1);
  //手机结果类型
  public static String phoneType = null;
  //弹框标识
  public static final int NOTIFICATION_RECEIVE = 0x102;
  //回调
  public CallbackContext mCllbackContext;


  public PushInit() {
    instance = this;
  }

  @Override
  public void initialize(CordovaInterface cordova, CordovaWebView webView) {
    super.initialize(cordova, webView);
    //初始化
    this.cordovaWebView = webView;
    this.cordovaActivity = cordova.getActivity();
    this.cordovaContext = cordova.getActivity().getApplicationContext();
    webView.getContext();
    MobilePhoneType.getXiaoMiKey(cordovaContext);
    MobilePhoneType.initRegister(cordovaContext);
  }

  @Override
  public boolean execute(final String action, final JSONArray args, final CallbackContext callbackContext) throws JSONException {
    Log.d(TAG, "-------------》execute args：" + args);
    this.mCllbackContext = callbackContext;
    if (!methodList.contains(action)) {
      return false;
    }
    if (action.equals("openSetingOfNotification")) {
      cordova.getThreadPool().execute(new Runnable() {
        @Override
        public void run() {
          toSetting();
        }
      });

    }
    threadPool.execute(new Runnable() {
      @Override
      public void run() {
        try {
          phoneType = MobilePhoneType.getType(cordovaActivity);
          Method method = PushInit.class.getDeclaredMethod(action, JSONArray.class, CallbackContext.class, String.class, CordovaInterface.class);
          method.invoke(PushInit.this, args, callbackContext, phoneType, cordova);
        } catch (NoSuchMethodException noSuchMethodException) {
          Log.d(TAG, noSuchMethodException.toString());
        } catch (Exception e) {
          Log.d(TAG, e.toString());
        }
      }
    });
    return true;
  }


   void areNotificationEnabled(JSONArray data, CallbackContext callbackContext, String phoneType, CordovaInterface mcordova) {
    int isEnabled;
    if (hasPermission("OP_POST_NOTIFICATION")) {
      isEnabled = 1;
    } else {
      isEnabled = 0;
    }
     callbackContext.success(isEnabled);
  }

  @TargetApi(Build.VERSION_CODES.KITKAT)
  private boolean hasPermission(String appOpsServiceId) {
    Context context = cordova.getActivity().getApplicationContext();
    AppOpsManager mAppOps = (AppOpsManager) context.getSystemService(Context.APP_OPS_SERVICE);
    ApplicationInfo appInfo = context.getApplicationInfo();

    String pkg = context.getPackageName();
    int uid = appInfo.uid;
    Class appOpsClazz = null;

    try {
      appOpsClazz = Class.forName(AppOpsManager.class.getName());
      Method checkOpNoThrowMethod = appOpsClazz.getMethod("checkOpNoThrow",
        Integer.TYPE, Integer.TYPE, String.class);
      Field opValue = appOpsClazz.getDeclaredField(appOpsServiceId);
      int value = opValue.getInt(Integer.class);
      Object result = checkOpNoThrowMethod.invoke(mAppOps, value, uid, pkg);

      return Integer.parseInt(result.toString()) == AppOpsManager.MODE_ALLOWED;
    } catch (InvocationTargetException e) {
      e.printStackTrace();
    } catch (IllegalAccessException e) {
      e.printStackTrace();
    } catch (NoSuchMethodException e) {
      e.printStackTrace();
    } catch (NoSuchFieldException e) {
      e.printStackTrace();
    } catch (ClassNotFoundException e) {
      e.printStackTrace();
    }
    return true;
  }

  private void toSetting() {
    Intent localIntent = new Intent();
    localIntent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
    if (Build.VERSION.SDK_INT >= 9) {
      localIntent.setAction("android.settings.APPLICATION_DETAILS_SETTINGS");
      localIntent.setData(Uri.fromParts("package", cordovaContext.getPackageName(), null));
    } else if (Build.VERSION.SDK_INT <= 8) {
      localIntent.setAction(Intent.ACTION_VIEW);
      localIntent.setClassName("com.android.settings", "com.android.setting.InstalledAppDetails");
      localIntent.putExtra("com.android.settings.ApplicationPkgName", cordovaContext.getPackageName());
    }
    cordovaActivity.startActivity(localIntent);
  }

  @Override
  public void onDestroy() {
    super.onDestroy();
    cordovaActivity = null;
    instance = null;
  }

  //初始化
  void init(JSONArray data, CallbackContext callbackContext, String phoneType, CordovaInterface mcordova) {
    if (phoneType == null) {
      return;
    }
    SpData spData = new SpData(cordovaActivity);
    //String type = spData.getStringValue("sys_type", null);
    String alert = spData.getStringValue(MobilePhoneType.HUATWEI_ALERT, null);
    String jsonExtras = spData.getStringValue(MobilePhoneType.HUATWEI_EXTRAS, null);

    JSONObject jsonObject = null;
    if (alert != null && jsonExtras != null) {
      try {
        //赋空
        spData.setStringValue(MobilePhoneType.HUATWEI_ALERT, null);
        spData.setStringValue(MobilePhoneType.HUATWEI_EXTRAS, null);

        jsonObject = new JSONObject(jsonExtras);
        JSONObject json = test(alert, jsonObject);
        //发送
        openNotificationReceive(json);
      } catch (JSONException e) {
        e.printStackTrace();
      }
    }
    // MobilePhoneType.initRegister(cordova.getActivity().getApplicationContext());

    //返回手机型号
    String type = MobilePhoneType.getType(cordovaActivity);
    JSONObject jsonType = new JSONObject();
    if (type != null) {
      try {
        jsonType.put("channelType", type);//渠道
        callbackContext.success(jsonType);
      } catch (JSONException e) {
        e.printStackTrace();
      }
    } else {
      Log.d(TAG, "null type");
    }
  }


  //设置别名
  void setAlias(JSONArray data, CallbackContext callbackContext, String phoneType, CordovaInterface mcordova) {
    if (data == null) {
      Log.d(TAG, "null alias");
      return;
    }
    SetAliasClass aliasClass = new SetAliasClass();
    aliasClass.SetAliasClass(mcordova, phoneType, data, callbackContext);
  }

  //设置标签
  void setTags(JSONArray data, CallbackContext callbackContext, String phoneType, CordovaInterface mcordova) {
    if (data == null) {
      Log.d(TAG, "null tags");
      return;
    }
    SetTagsClass tagsClass = new SetTagsClass();
    tagsClass.SetTagsClass(data, callbackContext, phoneType, mcordova);
  }


  /**
   * 返回唯一标识
   * 华为：toKen
   * 小米：mRegId
   * 极光：regId
   */

  //获取手机唯一识别码 Android id
  public void getUuid(JSONArray data, CallbackContext callbackContext, String phoneType, CordovaInterface mcordova) {

    String uniqueId = "";
    String type = MobilePhoneType.getSystem();
    if (type != null) {
      if (type.equals("sys_emui")) {
        //华为
        SpData spData = new SpData(cordovaActivity);
        uniqueId = spData.getStringValue("sys_huawei_regid", null);
      } else if (type.equals("sys_miui")) {
        //小米
        SpData spData = new SpData(cordovaActivity);
        uniqueId = spData.getStringValue("sys_mi_regid", null);
      } else {
        uniqueId = JPushInterface.getRegistrationID(cordovaActivity);
      }
    } else {
      uniqueId = JPushInterface.getRegistrationID(cordovaActivity);
    }
    try {
      JSONObject jsonObject = new JSONObject();
      jsonObject.put("pushTypeId", uniqueId);
      mCllbackContext.success(jsonObject);

    } catch (JSONException e) {
      e.printStackTrace();
      mCllbackContext.error("error json");
    }

  }


  /**
   * 华为   方法作废不使用
   */
  public void getHuaweiData(JSONArray data, CallbackContext callbackContext, String phoneType, CordovaInterface mcordova) {
    JSONObject jsonBack;
    SpData spData = new SpData(cordovaActivity);
    String alert = spData.getStringValue(MobilePhoneType.HUATWEI_ALERT, null);
    String jsonExtras = spData.getStringValue(MobilePhoneType.HUATWEI_EXTRAS, null);

    if (alert != null && jsonExtras != null) {
      try {
        JSONObject jsonObject = new JSONObject(jsonExtras);
        JSONObject json = test(alert, jsonObject);

        jsonBack = new JSONObject();
        jsonBack.put("json", json);
        //赋空
        spData.setStringValue(MobilePhoneType.HUATWEI_ALERT, null);
        spData.setStringValue(MobilePhoneType.HUATWEI_EXTRAS, null);
        //传递
        mCllbackContext.success(jsonBack);
      } catch (JSONException e) {
        mCllbackContext.error(e.toString());
      }
    }

  }

  /**
   * 华为数据需要再次处理转化为jsonobj对象
   */

  private static JSONObject jExtras;

  private static JSONObject test(String alert, JSONObject extras) {
    JSONObject jsonObject = new JSONObject();
    try {
      jsonObject.put(MobilePhoneType.ALERT, alert);
      if (extras != null) {

        jExtras = new JSONObject();

        String key;
        Iterator keys1 = extras.keys();
        while (keys1.hasNext()) {
          key = keys1.next().toString();
          jExtras.put(key, extras.getString(key));
        }
        jsonObject.put(MobilePhoneType.EXTRAS, jExtras);
      } else {
        jsonObject.put(MobilePhoneType.EXTRAS, null);
      }
    } catch (JSONException e) {
      e.printStackTrace();
    }

    return jsonObject;

  }


  /**
   * 接收通知
   *
   * @param extrasJson
   */
  public void obtainNotificationReceive(JSONObject extrasJson) {
    if (instance == null) {
      return;
    }
    PushProcessingClass.ReceiveNotice(cordovaActivity, instance, cordovaWebView, extrasJson);
  }

  /**
   * 打开通知
   *
   * @param extrasJson
   */
  public void openNotificationReceive(JSONObject extrasJson) {
    if (instance == null) {
      return;
    }
    PushProcessingClass.openReceiveNotice(cordovaActivity, instance, cordovaWebView, extrasJson);
  }

  /**
   * 消息透传
   *
   * @param extrasJson
   */
  public void messageTransmission(JSONObject extrasJson) {
    if (instance == null) {
      return;
    }
    PushProcessingClass.receiveMessage(cordovaActivity, instance, cordovaWebView, extrasJson);
  }
}
