package cordova.plugin.chief.push;

import android.app.Activity;
import android.app.ActivityManager;
import android.content.ComponentName;
import android.content.Context;
import android.util.Log;
import org.apache.cordova.CordovaWebView;
import org.json.JSONObject;
import java.util.List;

/**
 * 作者：隔壁骆叔叔 on 2017/6/29.
 * 邮箱：android_luochen@163.com
 *
 * 推送处理类：统一处理接收到的通知和消息 发送到前端JS
 */
public class PushProcessingClass {

  private static final  String TAG = "PushProcessingClass";

  //接受通知
  public static void ReceiveNotice(Activity mActivity, final PushInit instance,final CordovaWebView webView, JSONObject extrasJson){
    Log.d(TAG," --------->  ReceiveNotice");
    if (webView == null){
      return;
    }
    String format = "window.plugins.pushInit.receiveNotificationInAndroidCallback(%s);";
    final String js = String.format(format, extrasJson.toString());
    mActivity.runOnUiThread(new Runnable() {
      @Override
      public void run() {
        webView.loadUrl("javascript:" + js);
      }
    });

  }


  //打开通知
  public static void openReceiveNotice(Activity mActivity, final PushInit instance,final CordovaWebView webView, JSONObject extrasJson){

    Log.d(TAG," --------->  openReceiveNotice");
    if (webView == null){
      Log.d(TAG," --------->  openReceiveNotice");
      return;
    }
    Log.d(TAG," --------->  openReceiveNotice 222："+extrasJson);

    String format = "window.plugins.pushInit.openNotificationInAndroidCallback(%s);";
    final String js = String.format(format, extrasJson.toString());
    mActivity.runOnUiThread(new Runnable() {
      @Override
      public void run() {
        webView.loadUrl("javascript:" + js);
      }
    });

  }


  //消息透传
  public static void receiveMessage(Activity mActivity, final PushInit instance,final CordovaWebView webView, JSONObject extrasJson){
    Log.d(TAG," --------->  receiveMessage");
    if (webView == null){
      return;
    }
    String format = "window.plugins.pushInit.messageTransmissionInAndroidCallback(%s);";
    final String js = String.format(format, extrasJson.toString());
    mActivity.runOnUiThread(new Runnable() {
      @Override
      public void run() {
        webView.loadUrl("javascript:" + js);
      }
    });

  }


  /**
   * 判断程序是否在前台进程
   * @param context
   * @return
     */
  public static boolean isAppOnForeground(Context context){
    // 返回在设备上运行的应用程序进程的列表
    ActivityManager activityManager=(ActivityManager)context.getApplicationContext().getSystemService(Context.ACTIVITY_SERVICE);
    String packageName = context.getPackageName();
    Log.d(TAG," --------->  packageName:"+packageName);
    List<ActivityManager.RunningAppProcessInfo> appProcesses = activityManager.getRunningAppProcesses();
    if (appProcesses == null){
      return false;
    }
    for (ActivityManager.RunningAppProcessInfo appProcessInfo : appProcesses){
      // 此对象关联的进程名称
      if (appProcessInfo.processName.equals(packageName) && appProcessInfo.importance == ActivityManager.RunningAppProcessInfo.IMPORTANCE_FOREGROUND){
        return true;
      }
    }
    return false;
  }




//是否在前台运行,判断是否在前台运行 就判断栈顶的是否是这个包名
//  info.topActivity.getPackageName().equals("com.helloboxpuls")
//是否在后台运行,判断是否在后台运行 就判断栈中是否有这个包名
//  info.baseActivity.getPackageName().equals("com.helloboxpuls")

  public static boolean isAppRunning(Context context) {
    String packageName = context.getPackageName();
    String topActivityClassName=getTopActivityName(context);

    if (packageName!=null&&topActivityClassName!=null&&topActivityClassName.startsWith(packageName)) {
      return true;
    } else {
      return false;
    }
  }

  private static String getTopActivityName(Context context){
    String topActivityClassName=null;
    ActivityManager activityManager =
      (ActivityManager)(context.getSystemService(android.content.Context.ACTIVITY_SERVICE )) ;
    List<ActivityManager.RunningTaskInfo> runningTaskInfos = activityManager.getRunningTasks(1) ;
    if(runningTaskInfos != null){
      ComponentName f=runningTaskInfos.get(0).topActivity;
      topActivityClassName=f.getClassName();
    }
    return topActivityClassName;
  }



}
