package cordova.plugin.chief.push.pushclass;

import android.app.ActivityManager;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.os.Bundle;
import android.text.TextUtils;
import android.util.Log;


import org.json.JSONException;
import org.json.JSONObject;

import java.util.Arrays;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.logging.Logger;

import cn.jpush.android.api.JPushInterface;
import cordova.plugin.chief.push.PushInit;
import cordova.plugin.chief.push.SpData;

/**
 * 自定义接收器
 *
 * 如果不定义这个 Receiver，则：
 * 1) 默认用户会打开主界面
 * 2) 接收不到自定义消息
 */
public class JiGuangReceiver extends BroadcastReceiver {
  private static final String TAG = "JPush";
  private static final List<String> IGNORED_EXTRAS_KEYS =
    Arrays.asList(
      "cn.jpush.android.TITLE",
      "cn.jpush.android.MESSAGE",
      "cn.jpush.android.APPKEY",
      "cn.jpush.android.NOTIFICATION_CONTENT_TITLE"
    );


  //第一版
  private PushInit mPushInit;
  private static final List<String> BUNDLE_EXTRAS_KEYS = Arrays.asList("cn.jpush.android.ALERT","cn.jpush.android.NOTIFICATION_CONTENT_TITLE","cn.jpush.android.EXTRA");


  @Override
  public void onReceive(Context context, Intent intent) {
    try {

      Bundle bundle = intent.getExtras();
      mPushInit = PushInit.instance;
      //注册极光返回的唯一标识
      if (JPushInterface.ACTION_REGISTRATION_ID.equals(intent.getAction())) {
        String regId = bundle.getString(JPushInterface.EXTRA_REGISTRATION_ID);
        Log.d(TAG, "接收Registration Id : " + regId);

      }
      //接收到推送下来的自定义消息
      else if (JPushInterface.ACTION_MESSAGE_RECEIVED.equals(intent.getAction())) {
        handlingMessageReceive(intent);
      }
      //接收到推送下来的通知
      else if (JPushInterface.ACTION_NOTIFICATION_RECEIVED.equals(intent.getAction())) {
        int notifactionId = bundle.getInt(JPushInterface.EXTRA_NOTIFICATION_ID);
        Log.d(TAG, "[JiGuangReceiver] 接收到推送下来的通知/ 接收到推送下来的通知的ID: " + notifactionId);

        handlingNotificationReceive(context,intent);
      }
      //打开到推送下来的通知
      else if (JPushInterface.ACTION_NOTIFICATION_OPENED.equals(intent.getAction())) {
        Log.d(TAG, "[JiGuangReceiver] 用户点击打开了通知");
        handlingNotificationOpen(context,intent);
			/*//打开自定义的Activity
				Intent i = new Intent(context, OpenCustomAct.class);
				i.putExtras(bundle);
				//i.setFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
				i.setFlags(Intent.FLAG_ACTIVITY_NEW_TASK | Intent.FLAG_ACTIVITY_CLEAR_TOP );
				context.startActivity(i);*/
      }
      else if (JPushInterface.ACTION_RICHPUSH_CALLBACK.equals(intent.getAction())) {
        Log.d(TAG, "[JiGuangReceiver] 用户收到到RICH PUSH CALLBACK: " + bundle.getString(JPushInterface.EXTRA_EXTRA));
        //在这里根据 JPushInterface.EXTRA_EXTRA 的内容处理代码，比如打开新的Activity， 打开一个网页等..

      } else if(JPushInterface.ACTION_CONNECTION_CHANGE.equals(intent.getAction())) {
        boolean connected = intent.getBooleanExtra(JPushInterface.EXTRA_CONNECTION_CHANGE, false);
        Log.w(TAG, "[JiGuangReceiver]" + intent.getAction() +" connected state change to "+connected);
      } else {
        Log.d(TAG, "[JiGuangReceiver] Unhandled intent - " + intent.getAction());
      }
    } catch (Exception e){

    }

  }


  //接收到推送下来的通知
  private void handlingNotificationReceive(Context context, Intent intent) {
//    Intent launch = context.getPackageManager().getLaunchIntentForPackage(
//      context.getPackageName());
//    launch.addCategory(Intent.CATEGORY_LAUNCHER);
//    launch.setFlags(Intent.FLAG_ACTIVITY_NEW_TASK | Intent.FLAG_ACTIVITY_SINGLE_TOP);

    String title = intent.getStringExtra(JPushInterface.EXTRA_NOTIFICATION_TITLE);
    String alert = intent.getStringExtra(JPushInterface.EXTRA_ALERT);

    //第一版
    Map<String, Object> extrasMap  = getNotificationExtras(intent);
    JSONObject extrasObJetct =getNotificationObject(alert,extrasMap);
    if (mPushInit == null){
      mPushInit = new PushInit();
    }
    Log.v(TAG, "极光 --------objectData:"+extrasMap);
    mPushInit.obtainNotificationReceive(extrasObJetct);
  }



  //打开到推送下来的通知
  private void handlingNotificationOpen(Context context, Intent intent) {

    Intent intentStart = context.getPackageManager().getLaunchIntentForPackage(context.getPackageName());
    intent.setFlags(Intent.FLAG_ACTIVITY_NEW_TASK | Intent.FLAG_ACTIVITY_RESET_TASK_IF_NEEDED);
    context.startActivity(intentStart);


    String title = intent.getStringExtra(JPushInterface.EXTRA_NOTIFICATION_TITLE);
    String alert = intent.getStringExtra(JPushInterface.EXTRA_ALERT);
    Map<String, Object> extrasMap = getNotificationExtras(intent);
    JSONObject extrasObJetct = getNotificationObject(alert,extrasMap);
    Log.v(TAG, "极光 --------objectData:"+extrasMap);
    if (mPushInit == null){
      mPushInit = new PushInit();
    }
    mPushInit.openNotificationReceive(extrasObJetct);



/*
    Intent launch = context.getPackageManager().getLaunchIntentForPackage(
      context.getPackageName());
    launch.addCategory(Intent.CATEGORY_LAUNCHER);
    launch.setFlags(Intent.FLAG_ACTIVITY_NEW_TASK | Intent.FLAG_ACTIVITY_SINGLE_TOP);
    context.startActivity(launch);
*/

  }



  //接收到推送下来的自定义消息
  private void handlingMessageReceive(Intent intent) {
    String msg = intent.getStringExtra(JPushInterface.EXTRA_MESSAGE);
    String extra = intent.getStringExtra(JPushInterface.EXTRA_EXTRA);
    JSONObject  extrasObject  = getMessageObject(msg,extra);
    if (mPushInit == null){
      mPushInit = new PushInit();
    }
    mPushInit.messageTransmission(extrasObject);
  }


  private JSONObject jo;
  private JSONObject getMessageObject(String message, String extra) {
    JSONObject data = new JSONObject();
    try {
      data.put("message",message);
      if (extra == null){
        jo = new JSONObject();
        data.put("extras",jo);
        return data;
      }

      jo= new JSONObject(extra);
      JSONObject jExtras = new JSONObject();
      String key;
      Iterator keys = jo.keys();
      while (keys.hasNext()) {
        key = keys.next().toString();
        jExtras.put(key, jo.getString(key));
      }
      data.put("extras", jExtras);
    } catch (JSONException e) {
      e.printStackTrace();
    }
    return data;
  }







  //////////////////// 通知

  /**
   * 解析附加字段
   * @param intent
   * @return
   * */
  private Map<String, Object> getNotificationExtras(Intent intent) {
    Map<String, Object> extrasMap = new HashMap<String, Object>();

    for (String key : intent.getExtras().keySet()) {
      if (BUNDLE_EXTRAS_KEYS.contains(key)) {
        if (!TextUtils.isEmpty((String) intent.getExtras().get(key))){
          extrasMap.put(key,intent.getExtras().get(key));
        }
      }
    }
    return extrasMap;
  }

  /**
   * 将标题，内容，附加字段拼接未Jsonobject
   * @param alert
   * @param extras
   * @return
   */
  private static JSONObject getNotificationObject(String alert, Map<String, Object> extras) {
    JSONObject data = new JSONObject();
    try {
      data.put("alert", alert);
      JSONObject jExtras = new JSONObject();
      for (Map.Entry<String, Object> entry : extras.entrySet()){
        if (entry.getKey().contains("cn.jpush.android.EXTRA")){
          JSONObject jo;
          if (!TextUtils.isEmpty((String) entry.getValue())) {
            jo = new JSONObject((String) entry.getValue());
            String key;
            Iterator keys = jo.keys();
            while (keys.hasNext()){
              key = keys.next().toString();
              jExtras.put(key,jo.getString(key));
            }
          }
        }
      }
      data.put("extras",jExtras);
    } catch (JSONException e) {
      e.printStackTrace();
    }
    return data;
  }






  // 打印所有的 intent extra 数据
/*  private static String printBundle(Bundle bundle) {
    StringBuilder sb = new StringBuilder();
    for (String key : bundle.keySet()) {
      if (key.equals(JPushInterface.EXTRA_NOTIFICATION_ID)) {
        sb.append("\nkey:" + key + ", value:" + bundle.getInt(key));
      }else if(key.equals(JPushInterface.EXTRA_CONNECTION_CHANGE)){
        sb.append("\nkey:" + key + ", value:" + bundle.getBoolean(key));
      } else if (key.equals(JPushInterface.EXTRA_EXTRA)) {
        if (TextUtils.isEmpty(bundle.getString(JPushInterface.EXTRA_EXTRA))) {
          Log.i(TAG, "This message has no Extra data");
          continue;
        }

        try {
          JSONObject json = new JSONObject(bundle.getString(JPushInterface.EXTRA_EXTRA));
          Iterator<String> it =  json.keys();

          while (it.hasNext()) {
            String myKey = it.next().toString();
            sb.append("\nkey:" + key + ", value: [" +
              myKey + " - " +json.optString(myKey) + "]");
          }
        } catch (JSONException e) {
          Log.e(TAG, "Get message extra JSON error!");
        }

      } else {
        sb.append("\nkey:" + key + ", value:" + bundle.getString(key));
      }
    }
    return sb.toString();
  }*/






/*
  private static JSONObject getNotificationObjectTest(String title,String alert, Map<String, Object> extras) {
    JSONObject data = new JSONObject();
    try {
      data.put("title", title);
      data.put("alert", alert);


      JSONObject jExtras = new JSONObject();

      for (Map.Entry<String, Object> entry : extras.entrySet()) {
        if (entry.getKey().equals("cn.jpush.android.EXTRA")) {
          JSONObject jo;
          if (TextUtils.isEmpty((String) entry.getValue())) {
            jo = new JSONObject();
          } else {
            jo = new JSONObject((String) entry.getValue());
            String key;
            Iterator keys = jo.keys();
            while (keys.hasNext()) {
              key = keys.next().toString();
              jExtras.put(key, jo.getString(key));
            }
          }
          jExtras.put("cn.jpush.android.EXTRA", jo);
        } else {
          jExtras.put(entry.getKey(), entry.getValue());
        }
      }
      if (jExtras.length() > 0) {
        data.put("extras", jExtras);
      }
    } catch (JSONException e) {
      e.printStackTrace();
    }
    return data;
  }*/



  /**
   * 判断是在前台还是后台存货
   * @param context
   * @return
   * true  后台
   * false 前台
   */
  public static boolean isBackground(Context context) {
    ActivityManager activityManager = (ActivityManager) context.getSystemService(Context.ACTIVITY_SERVICE);
    List<ActivityManager.RunningAppProcessInfo> appProcesses = activityManager.getRunningAppProcesses();
    for (ActivityManager.RunningAppProcessInfo appProcess : appProcesses) {
      if (appProcess.processName.equals(context.getPackageName())) {
        if (appProcess.importance == ActivityManager.RunningAppProcessInfo.IMPORTANCE_BACKGROUND) {
          //Log.i("后台", appProcess.processName);
          return true;
        }else {
          //Log.i("前台", appProcess.processName);
          return false;
        }
      }
    }
    return false;
  }
}
