package cordova.plugin.chief.push.pushclass;

import android.annotation.SuppressLint;
import android.app.ActivityManager;
import android.content.Context;
import android.content.Intent;
import android.text.TextUtils;
import android.util.Log;
import com.xiaomi.mipush.sdk.ErrorCode;
import com.xiaomi.mipush.sdk.MiPushClient;
import com.xiaomi.mipush.sdk.MiPushCommandMessage;
import com.xiaomi.mipush.sdk.MiPushMessage;
import com.xiaomi.mipush.sdk.PushMessageReceiver;
import org.json.JSONException;
import org.json.JSONObject;
import java.text.SimpleDateFormat;
import java.util.Arrays;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import cordova.plugin.chief.push.MobilePhoneType;
import cordova.plugin.chief.push.PushInit;
import cordova.plugin.chief.push.SpData;

/**
 * 1、PushMessageReceiver 是个抽象类，该类继承了 BroadcastReceiver。<br/>
 * 2、需要将自定义的 DemoMessageReceiver 注册在 AndroidManifest.xml 文件中：
 * <pre>
 * {@code
 *  <receiver
 *      android:name="com.xiaomi.mipushdemo.DemoMessageReceiver"
 *      android:exported="true">
 *      <intent-filter>
 *          <action android:name="com.xiaomi.mipush.RECEIVE_MESSAGE" />
 *      </intent-filter>
 *      <intent-filter>
 *          <action android:name="com.xiaomi.mipush.MESSAGE_ARRIVED" />
 *      </intent-filter>
 *      <intent-filter>
 *          <action android:name="com.xiaomi.mipush.ERROR" />
 *      </intent-filter>
 *  </receiver>
 *  }</pre>
 * 3、DemoMessageReceiver 的 onReceivePassThroughMessage 方法用来接收服务器向客户端发送的透传消息。<br/>
 * 4、DemoMessageReceiver 的 onNotificationMessageClicked 方法用来接收服务器向客户端发送的通知消息，这个回调方法会在用户手动点击通知后触发。<br/>
 * 5、DemoMessageReceiver 的 onNotificationMessageArrived 方法用来接收服务器向客户端发送的通知消息，
 * 这个回调方法是在通知消息到达客户端时触发。另外应用在前台时不弹出通知的通知消息到达客户端也会触发这个回调函数。<br/>
 * 6、DemoMessageReceiver 的 onCommandResult 方法用来接收客户端向服务器发送命令后的响应结果。<br/>
 * 7、DemoMessageReceiver 的 onReceiveRegisterResult 方法用来接收客户端向服务器发送注册命令后的响应结果。<br/>
 * 8、以上这些方法运行在非 UI 线程中。
 *
 * @author mayixiang
 */
public class XiaoMiMessageReceiver extends PushMessageReceiver {

  private static String TAG = "XiaoMiMessageReceiver";

  private PushInit mPushInit;
  //通知
  private static final List<String> IGNORED_EXTRAS_KEYS = Arrays.asList("model_not_in", "source", "notify_foreground", "__planId__");
  //消息
  private static final List<String> IGNORED_MESSAGE_EXTRAS_KEYS = Arrays.asList("model_not_in", "source", "notify_foreground", "__planId__", "mat", "mrt");
  private String mRegId;
  private String mTopic;
  private String mAlias;
  private String mAccount;
  private String mStartTime;
  private String mEndTime;

  private Context mContext;
  private Context mContextReceiver;
  private String mainProcessName;

  //通知保存
  public static JSONObject miJsonObject = null;


  /**
   * 接受通知
   *
   * @param context
   * @param message msgTitle：标题 2
   *                msgContent：内容 2
   *                msgExtras：{model_not_in=,scorpio,MIX, source=op, notify_foreground=1, __planId__=154582}
   */
  @Override
  public void onNotificationMessageArrived(Context context, MiPushMessage message) {
    Log.v(TAG, "接收通知");
    Map<String, Object> extrasMap = getNotificationExtras(message);                                     //解析附加字段
    JSONObject extrasObject = getNotificationObject(message.getDescription(), extrasMap);               //转换数据类型
    if (mPushInit == null) {
      mPushInit = new PushInit();
    }

    mPushInit.obtainNotificationReceive(extrasObject);

  }


  //打开通知
  @Override
  public void onNotificationMessageClicked(final Context context, MiPushMessage message) {
    this.mContextReceiver = context;
    Log.v(TAG, "点击通知");

    //上报点击消息  MiPushClient.reportMessageClicked();
    // Log.v(TAG, "处理数据");


   /* Intent mIntent = new Intent(context, MainActivity.class);
    mIntent.setFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
    context.startActivity(mIntent);*/


    Map<String, Object> extrasMap = getNotificationExtras(message);//解析附加字段
    JSONObject extrasObject = getNotificationObject(message.getDescription(), extrasMap);//转换数据类型
    if (mPushInit == null) {
      mPushInit = new PushInit();
    }


    //如果进程存在
    if (isRunning(mContextReceiver)) {
      if (!isBackground(mContextReceiver)) {
        Log.d(TAG, "------------------------->前台存活");
        mPushInit.openNotificationReceive(extrasObject);
      } else if (isBackground(mContextReceiver)) {
        Log.e(TAG, "------------------------->后台存活");
        Intent mIntent = new Intent(context, MyActivity.class);
        mIntent.setFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
        context.startActivity(mIntent);
        mPushInit.openNotificationReceive(extrasObject);
      }
    } else {
      Log.e(TAG, "------------------------->重新启动");
      SpData spData = new SpData(mContextReceiver);
      spData.setStringValue(MobilePhoneType.HUATWEI_ALERT, message.getDescription());
      spData.setStringValue(MobilePhoneType.HUATWEI_EXTRAS, extrasObject.toString());
      // miJsonObject = extrasObject;
      Intent mIntent = new Intent(context, MyActivity.class);
      mIntent.setFlags(Intent.FLAG_ACTIVITY_NEW_TASK);

      context.startActivity(mIntent);
    }


  }


  /**
   * 判断是在前台还是后台存货
   *
   * @param context
   * @return true  后台
   * false 前台
   */
  public static boolean isBackground(Context context) {
    ActivityManager activityManager = (ActivityManager) context.getSystemService(Context.ACTIVITY_SERVICE);
    List<ActivityManager.RunningAppProcessInfo> appProcesses = activityManager.getRunningAppProcesses();
    for (ActivityManager.RunningAppProcessInfo appProcess : appProcesses) {
      if (appProcess.processName.equals(context.getPackageName())) {
        if (appProcess.importance == ActivityManager.RunningAppProcessInfo.IMPORTANCE_BACKGROUND) {
          //Log.i("后台", appProcess.processName);
          return true;
        } else {
          //Log.i("前台", appProcess.processName);
          return false;
        }
      }
    }
    return false;
  }

  /**
   * 应用是否在进程中
   *
   * @param context
   * @return true 在
   */
  private boolean isRunning(Context context) {
    ActivityManager am = (ActivityManager) context.getSystemService(Context.ACTIVITY_SERVICE);
    List<ActivityManager.RunningTaskInfo> list = am.getRunningTasks(100);
    boolean isAppRunning = false;
    //String MY_PKG_NAME = "com.cyberblue.iitag";
    String MY_PKG_NAME = context.getPackageName();
    for (ActivityManager.RunningTaskInfo info : list) {
      if (info.topActivity.getPackageName().equals(MY_PKG_NAME) || info.baseActivity.getPackageName().equals(MY_PKG_NAME)) {
        isAppRunning = true;
        Log.i(TAG, info.topActivity.getPackageName() + " info.baseActivity.getPackageName()=" + info.baseActivity.getPackageName());
        return isAppRunning;
      }
    }

    return isAppRunning;
  }


  @Override
  public void onCommandResult(Context context, MiPushCommandMessage message) {
    this.mContext = context;
    mainProcessName = context.getPackageName();
    this.mPushInit = new PushInit();
    Log.v(TAG, "onCommandResult----------------------》");
    String command1 = message.getCommand();
    List<String> arguments1 = message.getCommandArguments();
    String cmdArg11 = ((arguments1 != null && arguments1.size() > 0) ? arguments1.get(0) : null);
    String cmdArg2 = ((arguments1 != null && arguments1.size() > 1) ? arguments1.get(1) : null);
    String log;
    //是否注册成功
    if (MiPushClient.COMMAND_REGISTER.equals(command1)) {
      if (message.getResultCode() == ErrorCode.SUCCESS) {
        mRegId = cmdArg11;
        SpData spData = new SpData(context);
        spData.setStringValue("sys_mi_regid", mRegId);
        log = "注册成功";
      /*  if (mPushInit == null) {
          mPushInit = new PushInit();
        }
        mPushInit.upLoadUniqueId(mRegId);*/
      } else {
        log = "注册失败";
        new Thread() {
          @Override
          public void run() {
            super.run();
            try {
              MiPushClient.registerPush(mContext, MobilePhoneType.app_id, MobilePhoneType.app_key);
              sleep(3000);
            } catch (InterruptedException e) {
              e.printStackTrace();
            }
          }
        }.start();
      }

    }
    //设置别名
    else if (MiPushClient.COMMAND_SET_ALIAS.equals(command1)) {
      if (message.getResultCode() == ErrorCode.SUCCESS) {
        mAlias = cmdArg11;
        log = "别名设置成功";
      } else {
        log = "别名设置失败";
      }
    }
    //取消别名
    else if (MiPushClient.COMMAND_UNSET_ALIAS.equals(command1)) {
      if (message.getResultCode() == ErrorCode.SUCCESS) {
        mAlias = cmdArg11;
        log = "取消别名成功";
      } else {
        log = "取消别名失败";
      }
    }
    //设置账户
    else if (MiPushClient.COMMAND_SET_ACCOUNT.equals(command1)) {
      if (message.getResultCode() == ErrorCode.SUCCESS) {
        mAccount = cmdArg11;
        log = "设置账户成功";
      } else {
        log = "设置账户失败";
      }
    }
    //取消账户
    else if (MiPushClient.COMMAND_UNSET_ACCOUNT.equals(command1)) {
      if (message.getResultCode() == ErrorCode.SUCCESS) {
        mAccount = cmdArg11;
        log = "取消账户成功";
      } else {
        log = "取消账户失败";
      }
    }
    //设置订阅
    else if (MiPushClient.COMMAND_SUBSCRIBE_TOPIC.equals(command1)) {
      if (message.getResultCode() == ErrorCode.SUCCESS) {
        mTopic = cmdArg11;
        log = "设置订阅成功";
      } else {
        log = "设置订阅失败";
      }
    }
    //取消订阅
    else if (MiPushClient.COMMAND_UNSUBSCRIBE_TOPIC.equals(command1)) {
      if (message.getResultCode() == ErrorCode.SUCCESS) {
        mTopic = cmdArg11;
        log = "取消订阅成功";
      } else {
        log = "取消订阅失败";
      }
    }
    //设置时间段
    else if (MiPushClient.COMMAND_SET_ACCEPT_TIME.equals(command1)) {
      if (message.getResultCode() == ErrorCode.SUCCESS) {
        mStartTime = cmdArg11;
        mEndTime = cmdArg2;
        log = "设置时间段成功：" + mEndTime + "--" + mEndTime;

      } else {
        log = "设置时间段失败：" + mEndTime + "--" + mEndTime;
      }
    } else {
      log = message.getReason();
    }
    Log.v(TAG, "message.getReason()： " + message.getReason());

  }

  @Override
  public void onReceiveRegisterResult(Context context, MiPushCommandMessage message) {
    Log.v(TAG, "onReceiveRegisterResult is called. " + message.toString());
    String command = message.getCommand();
    List<String> arguments = message.getCommandArguments();
    String cmdArg1 = ((arguments != null && arguments.size() > 0) ? arguments.get(0) : null);
    String log;

    mPushInit = PushInit.instance;
    if (MiPushClient.COMMAND_REGISTER.equals(command)) {
      if (message.getResultCode() == ErrorCode.SUCCESS) {
        mRegId = cmdArg1;
        log = "Register push success. 注册成功";
        Log.e("注册成功", "注册成功");

      } else {
        log = "Register push fail. 注册失败";
        Log.e("注册失败", "注册失败");
      }
    } else {
      log = message.getReason();
    }

    Log.v(TAG, "log： " + log);
  }

  @SuppressLint("SimpleDateFormat")
  private static String getSimpleDate() {
    return new SimpleDateFormat("MM-dd hh:mm:ss").format(new Date());
  }


  //接收消息
  @Override
  public void onReceivePassThroughMessage(Context context, MiPushMessage message) {

    Map<String, Object> extraMap = getMessageExtras(message);
    JSONObject extrasObject = getMessageObject(message.getContent(), extraMap);
    Log.v(TAG, "接受透传消息 extraMap：" + extraMap.toString());
    Log.v(TAG, "接受透传消息 extrasObject：" + extrasObject.toString());
    if (mPushInit == null) {
      mPushInit = new PushInit();
    }
    mPushInit.messageTransmission(extrasObject);
  }

  private Map<String, Object> getMessageExtras(MiPushMessage miPushMessage) {
    Map<String, Object> extrasMap = new HashMap<String, Object>();
    for (String key : miPushMessage.getExtra().keySet()) {
      if (!IGNORED_MESSAGE_EXTRAS_KEYS.contains(key)) {
        extrasMap.put(key, miPushMessage.getExtra().get(key));
      }
    }
    return extrasMap;
  }

  private static JSONObject getMessageObject(String message, Map<String, Object> extras) {
    JSONObject data = new JSONObject();
    try {
      data.put("message", message);
      JSONObject jExtras = new JSONObject();
      for (Map.Entry<String, Object> entry : extras.entrySet()) {
        if (!TextUtils.isEmpty((String) entry.getValue())) {
          jExtras.put(entry.getKey(), entry.getValue());
        }
      }
      data.put("extras", jExtras);
    } catch (JSONException e) {
      e.printStackTrace();
    }
    return data;
  }

  /**
   * 解析转换收到的通知内容
   * {"alert":"推搡","title":"推搡","extras":"{sex=男, age=25, name=骆晨}"}
   * 转换
   * {"alert":"neironfg","title":"biaoti","extras":{"name":"{\"name1\":\"骆晨\"}","age":"18"}}
   *
   * @param alert
   * @param extras
   * @return
   */
  private static JSONObject getNotificationObject(String alert, Map<String, Object> extras) {
    JSONObject data = new JSONObject();
    try {
      data.put("alert", alert);
      JSONObject jExtras = new JSONObject();
      for (Map.Entry<String, Object> entry : extras.entrySet()) {
        if (!TextUtils.isEmpty((String) entry.getValue())) {
          jExtras.put(entry.getKey(), entry.getValue());
        }
      }
      data.put("extras", jExtras);
    } catch (JSONException e) {
      e.printStackTrace();
    }
    return data;
  }

  /**
   * 转换接受的通知    小米原始数据类型为 Map<String, String> 转 Map<String, Object>
   *
   * @param miPushMessage
   * @return
   */
  private Map<String, Object> getNotificationExtras(MiPushMessage miPushMessage) {
    Map<String, Object> extrasMap = new HashMap<String, Object>();
    for (String key : miPushMessage.getExtra().keySet()) {
      if (!IGNORED_EXTRAS_KEYS.contains(key)) {
        extrasMap.put(key, miPushMessage.getExtra().get(key));
      }
    }
    return extrasMap;
  }
}


